<?php
/**
 * File handler
 * @package    Albireo I
 * @subpackage Core
 * @copyright  Copyright (c) 2009 Albireo Solutions. All rights reserved
 * @license    GNU/AGPL http://www.gnu.org/licenses/agpl.html
 *
 * Albireo is free software! You can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *	GNU Affero General Public License for more details.
 */
class FileHandler {

    /**
     * validates file name form
     * @param String $file_name original file name
     * @return String validated file name
     */
    public static function validateFileName ( $file_name )
    {
        $bad_chars = array ( " ", "<", ">", "$", "^", "#", "@", "*", "\\", "(", ")", "%", "&", "?", ";", ",", "\'", "`", "\"", "á", "Á", "ä", "Ä", "č", "Č", "ď", "Ď", "é", "É", "ě", "ë", "Ě", "Ë", "í", "ï", "Í", "ĺ", "Ĺ", "ľ", "Ľ", "ň", "Ň", "ó", "Ó", "ö", "Ö", "ô", "ŕ", "Ŕ", "ř", "Ř", "š", "Š", "ť", "Ť", "ú", "Ú", "ů", "ü", "Ů", "ý", "Ý", "ž", "Ž" );
        $asci_chars = array ( "-", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "", "a", "A", "a", "A", "c", "C", "d", "D", "e", "E", "e", "e", "E", "E", "i", "I", "i", "l", "L", "l", "L", "n", "N", "o", "O", "o", "o", "O", "r", "R", "r", "R", "s", "S", "t", "T", "u", "U", "u", "u", "U", "y", "Y", "z", "Z" );
        $file = str_replace ( $bad_chars, $asci_chars, trim ( $file_name ) );
        return urlencode ( $file );
    }

    /**
     * removes target directory including its contents physically and also in database if needed.
     * Ignores all users access permissions.
     * Its possible that some files wont be removed physically due to server restrictions but they will be removed in database.
     * @param String $directory path to the target directory (should be relatively from Albireo root folder)
     * @param boolean $only_contents whether to delete only contents all the directory or the directory itself too
     * @param boolean $delete_in_database whether the method should delete corresponding database entries
     * @return mixed returns false if the directory doesn't exist or associative array:
     * "ok" => number of successfully removed objects,
     * "errors" => number of unsuccessfully removed objects,
     * "error messages" => array with all error messages
     */
    public static function removeDir ( $directory, $only_contents = false, $delete_in_database = false )
    {
	$result = array (
	    "ok" => 0,
	    "error" => 0,
	    "error messages" => array ()
	);
	if ( !file_exists ( $directory ) ) return false;
    	$folder = opendir ( $directory );
	$counter = 0;
    	while ( $file = readdir ( $folder ) ) {
	    if ( is_dir ( $directory . $file ) ) {
	    	if ( $file != "." && $file != ".."  ) {
		    $inner_result = self::removeDir ( $directory ."$file/" );
		    $result [ "ok" ] += $inner_result [ "ok" ];
		    $result [ "error" ] += $inner_result [ "error" ];
		    $result [ "error messages" ] = array_merge ( $result [ "error messages" ], $inner_result [ "error messages" ] );
		    if ( @rmdir ( $directory ."$file/" ) ) {
			++$result [ "ok" ];
		    } else {
			++$result [ "error" ];
			$result [ "error messages" ][] = _( "Složku nelze smazat:" ). ' '. $directory . $file .', '. _( "zápis:" ) .' '. substr ( @sprintf ( '%o', @fileperms ( $directory . $file ) ), -4 );
		    }
	    	}
	    } else {
	    	if ( @unlink ( $directory . $file ) ) {
		    ++$result [ "ok" ];
		} elseif ( file_exists ( $directory . $file ) ) {
		    ++$result [ "error" ];
		    $result [ "error messages" ][] = _( "Soubor nelze smazat:" ). ' '. $directory . $file .', '. _( "zápis:" ) .' '. substr ( @sprintf ( '%o', @fileperms ( $directory . $file ) ), -4 );
		}
	    }
	}
	closedir ( $folder );
	if ( !$only_contents && @rmdir ( $directory ) ) {
	    ++$result [ "ok" ];
	}
	if ( $delete_in_database ) {
	    $directory = substr($directory, 0, -1);
	    $last = strrpos($directory, "/");
	    $url = substr($directory, 0, $last);
	    $physicalname = substr($directory, $last);
	    $physicalname = substr($physicalname, 1);
	    if ( !Database::connection () -> query ( "DELETE FROM `drivefilesv2` WHERE (`url`='/$url/' AND `physicalname`='$physicalname') OR `url` LIKE '/$url/$physicalname/%';" ) ) {
		++$result [ "error" ];
		$result [ "error messages" ][] = _( "Nebylo možné smazat databázové záznamy objektů." );
	    }
	}
	return $result;
    }
}
?>